package test.listeners.github1029;

import org.testng.Assert;
import org.testng.Reporter;
import org.testng.annotations.Test;

public class Issue1029SampleTestClassWithFiveMethods {
    @Test
    public void a() {
        Assert.assertTrue(true);
    }
    @Test
    public void b() {
        Assert.assertTrue(true);
    }
    @Test
    public void c() {
        Assert.assertTrue(true);
    }
    @Test
    public void d() {
        Assert.assertTrue(true);
    }
    @Test
    public void e() {
        Assert.assertTrue(true);
    }

}
