package test.configuration.github1338;

import org.testng.annotations.Test;

public class SecondGitHub1338Sample extends BaseGitHub1338Sample {

  @Test(groups = {"group2"})
  public void secondTest() throws Exception {
  }
}
