package test.inheritance.github980;

import org.testng.Assert;
import org.testng.annotations.Test;

public class ParentClassSample {
    @Test
    public void a() {
        Assert.assertTrue(true);
    }

    @Test
    public void b() {
        Assert.assertTrue(true);
    }
}
